@@ -24,6 +24,8 @@
 #include <stdio.h>
 #include <unistd.h>
 
+#include <pulse/pulseaudio.h>
+
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -63,6 +65,7 @@ struct GvcChannelBarPrivate
         gboolean       symmetric;
         gboolean       click_lock;
         gboolean       is_amplified;
+        guint32        base_volume;
 };
 
 enum
@@ -143,7 +146,6 @@ _scale_box_new (GvcChannelBar *bar)
 
                 gtk_box_pack_start (GTK_BOX (sbox), priv->image, FALSE, FALSE, 0);
                 gtk_box_pack_start (GTK_BOX (sbox), priv->label, FALSE, FALSE, 0);
-
                 gtk_box_pack_start (GTK_BOX (box), priv->scale, TRUE, TRUE, 0);
 
                 bar->priv->end_box = ebox = gtk_hbox_new (FALSE, 6);
@@ -155,6 +157,7 @@ _scale_box_new (GvcChannelBar *bar)
         }
 
         gtk_range_set_update_policy (GTK_RANGE (priv->scale), GTK_UPDATE_CONTINUOUS);
+        gtk_scale_set_digits (GTK_SCALE (priv->scale), 0);
         ca_gtk_widget_disable_sounds (bar->priv->scale, FALSE);
         gtk_widget_add_events (bar->priv->scale, GDK_SCROLL_MASK);
 
@@ -394,6 +397,7 @@ on_scale_button_release_event (GtkWidget      *widget,
                                GdkEventButton *event,
                                GvcChannelBar  *bar)
 {
+        GtkAdjustment *adj;
         gdouble value;
 
         /* HACK: see on_scale_button_press_event() */
@@ -402,13 +406,16 @@ on_scale_button_release_event (GtkWidget      *widget,
 
         bar->priv->click_lock = FALSE;
 
-        value = gtk_adjustment_get_value (bar->priv->zero_adjustment);
-        gtk_adjustment_set_value (bar->priv->adjustment, value);
+        adj = gtk_range_get_adjustment (GTK_RANGE (widget));
+
+        value = gtk_adjustment_get_value (adj);
 
         /* this means the adjustment moved away from zero and
           therefore we should unmute and set the volume. */
-
-        gvc_channel_bar_set_is_muted (bar, FALSE);
+        if (value > 0)
+                gvc_channel_bar_set_is_muted (bar, FALSE);
+        else
+                gvc_channel_bar_set_is_muted (bar, TRUE);
 
         /* Play a sound! */
         ca_gtk_play_for_widget (GTK_WIDGET (bar), 0,
@@ -420,30 +427,53 @@ on_scale_button_release_event (GtkWidget      *widget,
         return FALSE;
 }
 
+gboolean
+gvc_channel_bar_scroll (GvcChannelBar *bar, GdkScrollDirection direction)
+{
+        GtkAdjustment *adj;
+        gdouble value;
+
+        g_return_val_if_fail (bar != NULL, FALSE);
+        g_return_val_if_fail (GVC_IS_CHANNEL_BAR (bar), FALSE);
+
+        /* FIXME we should handle left/right for horizontal bars */
+        if (direction != GDK_SCROLL_UP && direction != GDK_SCROLL_DOWN)
+                return FALSE;
+
+        adj = gtk_range_get_adjustment (GTK_RANGE (bar->priv->scale));
+        if (adj == bar->priv->zero_adjustment) {
+                if (direction == GDK_SCROLL_UP)
+                        gvc_channel_bar_set_is_muted (bar, FALSE);
+                return TRUE;
+        }
+
+        value = gtk_adjustment_get_value (adj);
+
+        if (direction == GDK_SCROLL_UP) {
+                if (value + ADJUSTMENT_MAX/100.0 > ADJUSTMENT_MAX)
+                        value = ADJUSTMENT_MAX;
+                else
+                        value = value + ADJUSTMENT_MAX/100.0;
+        } else if (direction == GDK_SCROLL_DOWN) {
+                if (value - ADJUSTMENT_MAX/100.0 < 0)
+                        value = 0.0;
+                else
+                        value = value - ADJUSTMENT_MAX/100.0;
+        }
+
+        gvc_channel_bar_set_is_muted (bar, (value == 0.0));
+        adj = gtk_range_get_adjustment (GTK_RANGE (bar->priv->scale));
+        gtk_adjustment_set_value (adj, value);
+
+        return TRUE;
+}
+
 static gboolean
 on_scale_scroll_event (GtkWidget      *widget,
                        GdkEventScroll *event,
                        GvcChannelBar  *bar)
 {
-        gdouble value;
-
-        value = gtk_adjustment_get_value (bar->priv->adjustment);
-
-        if (event->direction == GDK_SCROLL_UP) {
-                if (value + ADJUSTMENT_MAX/100.0 > ADJUSTMENT_MAX)
-                        value = ADJUSTMENT_MAX;
-                else
-                        value = value + ADJUSTMENT_MAX/100.0;
-                gtk_adjustment_set_value (bar->priv->adjustment, value);
-        } else if (event->direction == GDK_SCROLL_DOWN) {
-                if (value - ADJUSTMENT_MAX/100.0 < 0)
-                        value = 0.0;
-                else
-                        value = value - ADJUSTMENT_MAX/100.0;
-                gtk_adjustment_set_value (bar->priv->adjustment, value);
-        }
-
-        return TRUE;
+        return gvc_channel_bar_scroll (bar, event->direction);
 }
 
 static void
@@ -544,6 +574,47 @@ gvc_channel_bar_set_is_amplified (GvcChannelBar *bar, gboolean amplified)
         bar->priv->is_amplified = amplified;
         gtk_adjustment_set_upper (bar->priv->adjustment, ADJUSTMENT_MAX);
         gtk_adjustment_set_upper (bar->priv->zero_adjustment, ADJUSTMENT_MAX);
+        gtk_scale_clear_marks (GTK_SCALE (bar->priv->scale));
+
+        if (amplified) {
+                char *str;
+
+                if (bar->priv->base_volume == ADJUSTMENT_MAX_NORMAL) {
+                        str = g_strdup_printf ("<small>%s</small>", C_("volume", "100%"));
+                        gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), ADJUSTMENT_MAX_NORMAL,
+                                            GTK_POS_BOTTOM, str);
+                } else {
+                        str = g_strdup_printf ("<small>%s</small>", C_("volume", "Unamplified"));
+                        gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), bar->priv->base_volume,
+                                            GTK_POS_BOTTOM, str);
+                }
+
+                g_free (str);
+                gtk_alignment_set (GTK_ALIGNMENT (bar->priv->mute_box), 0.5, 0, 0, 0);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->low_image), 0.5, 0);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->high_image), 0.5, 0);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->label), 0, 0);
+        } else {
+                gtk_alignment_set (GTK_ALIGNMENT (bar->priv->mute_box), 0.5, 0.5, 0, 0);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->low_image), 0.5, 0.5);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->high_image), 0.5, 0.5);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->label), 0, 0.5);
+        }
+}
+
+void
+gvc_channel_bar_set_base_volume (GvcChannelBar *bar,
+                                 pa_volume_t    base_volume)
+{
+        g_return_if_fail (GVC_IS_CHANNEL_BAR (bar));
+
+        if (base_volume == 0) {
+                bar->priv->base_volume = ADJUSTMENT_MAX_NORMAL;
+                return;
+	}
+
+        /* Note that you need to call _is_amplified() afterwards to update the marks */
+        bar->priv->base_volume = base_volume;
 }
 
 static void
@@ -743,6 +814,7 @@ gvc_channel_bar_init (GvcChannelBar *bar)
 
         bar->priv = GVC_CHANNEL_BAR_GET_PRIVATE (bar);
 
+        bar->priv->base_volume = ADJUSTMENT_MAX_NORMAL;
         bar->priv->low_icon_name = g_strdup ("audio-volume-low");
         bar->priv->high_icon_name = g_strdup ("audio-volume-high");
 
@@ -788,9 +860,7 @@ gvc_channel_bar_init (GvcChannelBar *bar)
         gtk_widget_set_no_show_all (bar->priv->image, TRUE);
 
         bar->priv->label = gtk_label_new (NULL);
-        gtk_misc_set_alignment (GTK_MISC (bar->priv->label),
-                                0.0,
-                                0.5);
+        gtk_misc_set_alignment (GTK_MISC (bar->priv->label), 0.0, 0.5);
         gtk_widget_set_no_show_all (bar->priv->label, TRUE);
 
         /* frame */
