@@ -27,32 +27,38 @@
 #include <config.h>
 #endif
 
-#include <gconf/gconf-client.h>
-#include <gnome.h>
+#include <string.h>
 
+#include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
 #include <gst/gst.h>
 
 #include "gsr-window.h"
 
+void gsr_quit (void);
+void gsr_add_recent (gchar *filename);
+GtkWidget * gsr_open_window (const char *filename);
+
 extern void gnome_media_profiles_init (GConfClient *conf);
  
 static GList *windows = NULL;
 
+/* Also used in gsr-window.c as extern GConfClient *gconf_client */
 GConfClient *gconf_client = NULL;
 
 static gboolean
 delete_event_cb (GSRWindow *window,
 		 gpointer data)
 {
-	if (! gsr_window_is_saved (window)) {
-		close_confirmation_dialog (window);
+	if (!gsr_window_is_saved (window) && gsr_discard_confirmation_dialog (window, TRUE)) {
+		gsr_window_close (window);
 		return TRUE;
 	}
 
 	return FALSE;
 }
 
-void
+static void
 window_destroyed (GtkWidget *window,
 		  gpointer data)
 {
@@ -76,9 +82,7 @@ gsr_quit (void)
 		   p will be invalid */
 		p = p->next;
 
-		if (! gsr_window_is_saved (window))
-			close_confirmation_dialog (window);
-		else
+		if (gsr_window_is_saved (window) || gsr_discard_confirmation_dialog (window, TRUE))
 			gsr_window_close (window);
 	}
 }
@@ -121,6 +125,7 @@ gsr_add_recent (gchar *filename)
 
 }
 
+/* Also referenced from gsr-window.c */
 gint gsr_sample_count = 1;
 
 GtkWidget *
@@ -177,7 +182,7 @@ main (int argc,
 	};
 
 	GOptionContext *ctx;
-	GnomeProgram *program;
+	GError *error = NULL;
 
 	g_thread_init (NULL);
 
@@ -187,16 +192,19 @@ main (int argc,
 	textdomain (GETTEXT_PACKAGE);
 
 	ctx = g_option_context_new ("gnome-sound-recorder");
+	/* Initializes gtk during option parsing */
+	g_option_context_add_group (ctx, gtk_get_option_group (TRUE));
 	g_option_context_add_group (ctx, gst_init_get_option_group ());
 	g_option_context_add_main_entries (ctx, entries, GETTEXT_PACKAGE);
-	program = gnome_program_init ("gnome-sound-recorder", VERSION,
-	                              LIBGNOMEUI_MODULE, argc, argv,
-	                              GNOME_PARAM_GOPTION_CONTEXT, ctx,
-	                              GNOME_PARAM_HUMAN_READABLE_NAME,
-	                              "GNOME Sound Recorder",
-	                              GNOME_PARAM_APP_DATADIR, DATADIR,
-	                              NULL);
 
+	if (!g_option_context_parse (ctx, &argc, &argv, &error)) {
+		g_printerr ("Option parsing failed: %s\n", error->message);
+		g_error_free (error);
+		g_option_context_free (ctx);
+		return EXIT_FAILURE;
+	}
+
+	g_option_context_free (ctx);
 	gtk_window_set_default_icon_name ("gnome-sound-recorder");
 
 	/* use it like a singleton */
